@@ -28,7 +28,7 @@
 #include "calls-provider.h"
 
 #include <glib-object.h>
-#include <libpeas/peas.h>
+#include <libpeas.h>
 
 G_BEGIN_DECLS
 
