@@ -22,6 +22,10 @@
  *
  */
 
+#define WAIT_FOR_SHUTDOWN_MS 100
+#define WAIT_FOR_OFFLINE_MS 100
+#define WAIT_FOR_OFFLINE_RETRIES 5
+
 #define G_LOG_DOMAIN "CallsSipOrigin"
 
 
@@ -214,7 +218,8 @@ remove_calls (CallsSipOrigin *self,
 
   g_hash_table_remove_all (self->call_handles);
 
-  g_clear_pointer (&self->oper->call_handle, nua_handle_unref);
+  if (self->oper)
+    g_clear_pointer (&self->oper->call_handle, nua_handle_unref);
 }
 
 
@@ -1230,7 +1235,7 @@ deinit_sip_account (CallsSipOrigin *self)
   remove_calls (self, NULL);
 
   if (self->nua) {
-    g_debug ("Clearing any handles");
+    g_debug ("Clearing any handles for account '%s'", self->address);
     g_clear_pointer (&self->oper->register_handle, nua_handle_destroy);
     g_debug ("Requesting nua_shutdown ()");
     self->is_nua_shutdown = FALSE;
@@ -1238,7 +1243,7 @@ deinit_sip_account (CallsSipOrigin *self)
     nua_shutdown (self->nua);
     // need to wait for nua_r_shutdown event before calling nua_destroy ()
     while (!self->is_nua_shutdown)
-      su_root_step (self->ctx->root, 100);
+      su_root_step (self->ctx->root, WAIT_FOR_SHUTDOWN_MS);
 
     if (!self->is_shutdown_success) {
       g_warning ("nua_shutdown() timed out. Cannot proceed");
@@ -1248,8 +1253,7 @@ deinit_sip_account (CallsSipOrigin *self)
       return FALSE;
     }
     g_debug ("nua_shutdown() complete. Destroying nua handle");
-    nua_destroy (self->nua);
-    self->nua = NULL;
+    g_clear_pointer (&self->nua, nua_destroy);
   }
 
   g_clear_pointer (&self->own_ip, g_free);
@@ -1263,8 +1267,12 @@ deinit_sip_account (CallsSipOrigin *self)
 static void
 recreate_sip (CallsSipOrigin *self)
 {
-  if (deinit_sip_account (self))
-    init_sip_account (self, NULL);
+  if (deinit_sip_account (self)) {
+    g_autoptr (GError) err = NULL;
+
+    if (!init_sip_account (self, &err))
+      g_warning ("Failed to re-init sip account: %s", err->message);
+  }
 }
 
 
@@ -1506,6 +1514,21 @@ static void
 calls_sip_origin_dispose (GObject *object)
 {
   CallsSipOrigin *self = CALLS_SIP_ORIGIN (object);
+  gboolean su_step_timeout = FALSE;
+  guint i = 0;
+
+  if (!self->use_direct_connection && self->state == CALLS_ACCOUNT_STATE_ONLINE) {
+    go_online (CALLS_ACCOUNT (self), FALSE);
+
+    while (self->state != CALLS_ACCOUNT_STATE_OFFLINE && !su_step_timeout) {
+      su_root_step (self->ctx->root, WAIT_FOR_OFFLINE_MS);
+
+      if (i++ > WAIT_FOR_OFFLINE_RETRIES)
+        su_step_timeout = TRUE;
+    }
+  }
+
+  deinit_sip_account (self);
 
   g_clear_pointer (&self->id, g_free);
   g_clear_pointer (&self->own_ip, g_free);
@@ -1514,11 +1537,7 @@ calls_sip_origin_dispose (GObject *object)
   g_clear_pointer (&self->host, g_free);
   g_clear_pointer (&self->user, g_free);
   g_clear_pointer (&self->password, g_free);
-
-  if (!self->use_direct_connection && self->state == CALLS_ACCOUNT_STATE_ONLINE)
-    go_online (CALLS_ACCOUNT (self), FALSE);
-
-  deinit_sip_account (self);
+  g_clear_pointer (&self->address, g_free);
 
   G_OBJECT_CLASS (calls_sip_origin_parent_class)->dispose (object);
 }
